@@ -30,10 +30,11 @@ except ImportError:
     pep8 = None
 
 
-__version__ = '0.5.2'
+__version__ = '0.6'
 
 
 pep8bin = 'pep8'
+PEP8_PASSES_MAX = 100
 CR = '\r'
 LF = '\n'
 CRLF = '\r\n'
@@ -82,22 +83,12 @@ class FixPEP8(object):
         self.options = options
         self.indent_word = _get_indentword("".join(self.source))
         # method definition
+        self.fix_e111 = self.fix_e101
         self.fix_e222 = self.fix_e221
         self.fix_e223 = self.fix_e221
         self.fix_e241 = self.fix_e221
         self.fix_e242 = self.fix_e224
-
-    def _get_indentlevel(self, line):
-        sio = StringIO(line)
-        indent_word = ""
-        for t in tokenize.generate_tokens(sio.readline):
-            if t[0] == token.INDENT:
-                indent_word = t[1]
-                break
-        import math
-        indent_level = int(math.ceil(float(len(indent_word)) /
-                                     len(self.indent_word)))
-        return indent_level + 1
+        self.fix_w191 = self.fix_e101
 
     def _spawn_pep8(self, targetfile):
         """execute pep8 via subprocess.Popen."""
@@ -131,7 +122,7 @@ class FixPEP8(object):
 
     def _fix_source(self):
         completed_lines = []
-        for result in self.results:
+        for result in sorted(self.results, key=_priority_key):
             if result['line'] in completed_lines:
                 continue
             fixed_methodname = "fix_%s" % result['id'].lower()
@@ -140,6 +131,10 @@ class FixPEP8(object):
                 modified_lines = fix(result)
                 if modified_lines:
                     completed_lines += modified_lines
+                if self.options.verbose:
+                    if modified_lines == []:
+                        sys.stderr.write('Not fixing {f} on line {l}\n'.format(
+                                f=result['id'], l=result['line']))
                 completed_lines.append(result['line'])
             else:
                 if self.options.verbose:
@@ -152,9 +147,9 @@ class FixPEP8(object):
                                                         info))
 
     def _fix_whitespace(self, result, pattern, repl):
-        target = self.source[result['line'] - 1]
-        fixed = re.sub(pattern, repl, target)
-        self.source[result['line'] - 1] = fixed
+        (indent, rest) = _split_indentation(self.source[result['line'] - 1])
+        fixed = re.sub(pattern, repl, rest)
+        self.source[result['line'] - 1] = indent + fixed
 
     def fix(self):
         if pep8:
@@ -165,39 +160,14 @@ class FixPEP8(object):
         self._fix_source()
         return "".join(self.source)
 
-    #def fix_e101(self, result):
-    #    target = self.source[result['line'] - 1]
-    #    offset = result['column'] - 1
-    #    if target[offset] == '\t':
-    #        fixed = self.indent_word + target[offset + 1:]
-    #    else:
-    #        # FIXME: not implement
-    #        fixed = target
-    #    self.source[result['line'] - 1] = fixed
-
-    def fix_e111(self, result):
-        sio = StringIO("".join(self.source[result['line'] - 1:]))
-        last_line = ""
-        diff_cnt = 0
-        fixed_lines = []
-        for tokens in tokenize.generate_tokens(sio.readline):
-            if tokens[0] == token.INDENT:
-                _level = self._get_indentlevel(tokens[4])
-                diff_cnt = 4 * (_level - 1) - len(tokens[1])
-            if tokens[0] == token.DEDENT:
-                break
-            if tokens[4] != last_line:
-                last_line = tokens[4]
-                if diff_cnt >= 0:
-                    fixed_lines.append(" " * diff_cnt + tokens[4])
-                else:
-                    fixed_lines.append(tokens[4][abs(diff_cnt):])
-        for offset, fixed_line in enumerate(fixed_lines):
-            self.source[result['line'] - 1 + offset] = fixed_line
-
-        # Mark everything as modified since we don't want other instances of
-        # E111 fixes to interfere with this fix.
-        return range(len(self.source))
+    def fix_e101(self, _):
+        """Reindent all lines."""
+        reindenter = Reindenter(self.source)
+        if reindenter.run():
+            self.source = reindenter.fixed_lines()
+            return range(len(self.source))
+        else:
+            return []
 
     def fix_e201(self, result):
         self._fix_whitespace(result, r"\(\s+", "(")
@@ -230,7 +200,8 @@ class FixPEP8(object):
 
     def fix_e224(self, result):
         target = self.source[result['line'] - 1]
-        fixed = re.sub(r'\t+', ' ', target, 1)
+        offset = result['column'] - 1
+        fixed = target[:offset] + target[offset:].replace('\t', ' ')
         self.source[result['line'] - 1] = fixed
 
     def fix_e225(self, result):
@@ -238,9 +209,13 @@ class FixPEP8(object):
         offset = result['column']
         fixed = target[:offset - 1] + " " + target[offset - 1:]
 
-        # Only proceed if non-whitespace characters match
-        if fixed.replace(' ', '') == target.replace(' ', ''):
+        # Only proceed if non-whitespace characters match.
+        # And make sure we don't break the indentation.
+        if (fixed.replace(' ', '') == target.replace(' ', '') and
+            _get_indentation(fixed) == _get_indentation(target)):
             self.source[result['line'] - 1] = fixed
+        else:
+            return []
 
     def fix_e231(self, result):
         target = self.source[result['line'] - 1]
@@ -323,7 +298,7 @@ class FixPEP8(object):
         target = self.source[line_index]
 
         # Take care of semicolons first
-        if ';' in target:
+        if target.rstrip().endswith(';'):
             self.source[line_index] = _fix_multiple_statements(target,
                                                                self.newline)
         else:
@@ -334,15 +309,23 @@ class FixPEP8(object):
             self.source[line_index] = self.newline.join(fixed_modulelist)
 
     def fix_e701(self, result):
-        target = self.source[result['line'] - 1]
+        line_index = result['line'] - 1
+        target = self.source[line_index]
         c = result['column']
-        indent_level = self._get_indentlevel(target)
+
         fixed_source = target[:c] + self.newline + \
-                       self.indent_word * indent_level + target[c:].lstrip()
+                       _get_indentation(target) + self.indent_word + \
+                       target[c:].lstrip()
         self.source[result['line'] - 1] = fixed_source
 
     def fix_e702(self, result):
         target = self.source[result['line'] - 1]
+        if '"""' in target:
+            # We currently do not support things like
+            #     """
+            #         hello
+            #       """; foo()
+            return []
         self.source[result['line'] - 1] = _fix_multiple_statements(target,
                 self.newline)
 
@@ -421,7 +404,12 @@ class FixPEP8(object):
         line_index = result['line'] - 1
         line = self.source[line_index]
 
-        if ';' in line:
+        split_line = line.split(',')
+        if len(split_line) > 1 and split_line[1].strip().startswith('('):
+            # Give up
+            return []
+
+        if line.rstrip().endswith(';'):
             # Take care of semicolons first
             self.source[line_index] = _fix_multiple_statements(line,
                                                                self.newline)
@@ -476,14 +464,14 @@ class FixPEP8(object):
                 modified_lines.append(i)
             line = self.source[line_index]
 
-        # Skip cases with multiple arguments as to not handle tracebacks
-        # incorrectly in cases such as "raise Exception, Value, Traceback".
-        indent = (self._get_indentlevel(line) - 1) * self.indent_word
-        indent_offset = (self._get_indentlevel(line) - 1) * \
-                len(self.indent_word)
-        ast_body = ast.parse(line[indent_offset:]).body[0]
+        indent, rest = _split_indentation(line)
+        try:
+            ast_body = ast.parse(rest).body[0]
+        except SyntaxError:
+            # Give up
+            return []
+
         if len(ast_body._fields) == 3 and ast_body.tback is not None:
-            # TODO: Don't Care corner cases.
             _id = [indent, ]
             for node in ast.iter_child_nodes(ast_body):
                 if ast.Str == type(node):
@@ -496,7 +484,12 @@ class FixPEP8(object):
                 if ast.Name == type(node):
                     _id.append(node.id)
                     continue
-                _id.append(repr(ast.literal_eval(node)))
+                try:
+                    _id.append(repr(ast.literal_eval(node)))
+                except ValueError:
+                    # Give up
+                    return []
+
             # find space and comment
             sio = StringIO(line)
             old_tokens = None
@@ -509,10 +502,15 @@ class FixPEP8(object):
                     _id.append(self.newline)
                     break
                 old_tokens = tokens
-            # create to fixed source
-            self.source[result['line'] - 1] = "%sraise %s(%s), None, %s%s" % (
-                    tuple(_id))
-            return modified_lines
+            # Create fixed line and check for correctness
+            candidate = "%sraise %s(%s), None, %s%s" % tuple(_id)
+            pattern = '[)(, ]'
+            if (re.sub(pattern, repl='', string=candidate).replace('None', '')
+                    == re.sub(pattern, repl='', string=line)):
+                self.source[result['line'] - 1] = candidate
+                return modified_lines
+            else:
+                return []
 
         sio = StringIO(line)
         is_found_raise = False
@@ -562,7 +560,10 @@ class FixPEP8(object):
 
         # We do not support things like
         #     ``1`` + ``1``
-        if len(re.findall('`+', target)) > 2:
+        # And we do not support multiple lines like
+        #     `(1
+        #      )`
+        if len(re.findall('`+', target)) != 2:
             return
 
         start = target.find('`')
@@ -601,10 +602,20 @@ def _get_indentword(source):
     return indent_word
 
 
+def _get_indentation(line):
+    non_whitespace_index = len(line) - len(line.lstrip())
+    return line[:non_whitespace_index]
+
+
+def _split_indentation(line):
+    """Split into tuple (indentation, rest)."""
+    non_whitespace_index = len(line) - len(line.lstrip())
+    return (line[:non_whitespace_index], line[non_whitespace_index:])
+
+
 def _fix_multiple_statements(target, newline):
-    non_whitespace_index = len(target) - len(target.lstrip())
-    indentation = target[:non_whitespace_index]
-    f = [indentation + t.strip() for t in target.split(";") if t.strip()]
+    f = [_get_indentation(target) + t.strip()
+         for t in target.split(";") if t.strip()]
     return newline.join(f) + newline
 
 
@@ -625,6 +636,171 @@ def _get_difftext(old, new, filename):
     return "".join(difftext)
 
 
+def _priority_key(pep8_result):
+    """Key for sorting PEP8 results.
+    Global fixes should be done first. This is important for things
+    like indentation.
+    """
+    high_priority = ['e101', 'e111', 'w191']
+    return pep8_result['id'].lower() not in high_priority
+
+
+class Reindenter:
+    """Reindents badly-indented code to uniformly use four-space indentation.
+    Released to the public domain, by Tim Peters, 03 October 2000."""
+    def __init__(self, input_text):
+        self.find_stmt = 1  # next token begins a fresh stmt?
+        self.level = 0  # current indent level
+
+        # Raw file lines.
+        self.raw = input_text
+        self.after = None
+
+        # File lines, rstripped & tab-expanded.  Dummy at start is so
+        # that we can use tokenize's 1-based line numbering easily.
+        # Note that a line is all-blank iff it's "\n".
+        self.lines = [line.rstrip('\n \t').expandtabs() + "\n"
+                      for line in self.raw]
+        self.lines.insert(0, None)
+        self.index = 1  # index into self.lines of next line
+
+        # List of (lineno, indentlevel) pairs, one for each stmt and
+        # comment line.  indentlevel is -1 for comment lines, as a
+        # signal that tokenize doesn't know what to do about them;
+        # indeed, they're our headache!
+        self.stats = []
+
+    def run(self):
+        tokens = tokenize.generate_tokens(self.getline)
+        for t in tokens:
+            self.tokeneater(*t)
+        # Remove trailing empty lines.
+        lines = self.lines
+        while lines and lines[-1] == "\n":
+            lines.pop()
+        # Sentinel.
+        stats = self.stats
+        stats.append((len(lines), 0))
+        # Map count of leading spaces to # we want.
+        have2want = {}
+        # Program after transformation.
+        after = self.after = []
+        # Copy over initial empty lines -- there's nothing to do until
+        # we see a line with *something* on it.
+        i = stats[0][0]
+        after.extend(lines[1:i])
+        for i in range(len(stats) - 1):
+            thisstmt, thislevel = stats[i]
+            nextstmt = stats[i + 1][0]
+            have = _getlspace(lines[thisstmt])
+            want = thislevel * 4
+            if want < 0:
+                # A comment line.
+                if have:
+                    # An indented comment line.  If we saw the same
+                    # indentation before, reuse what it most recently
+                    # mapped to.
+                    want = have2want.get(have, - 1)
+                    if want < 0:
+                        # Then it probably belongs to the next real stmt.
+                        for j in range(i + 1, len(stats) - 1):
+                            jline, jlevel = stats[j]
+                            if jlevel >= 0:
+                                if have == _getlspace(lines[jline]):
+                                    want = jlevel * 4
+                                break
+                    if want < 0:           # Maybe it's a hanging
+                                           # comment like this one,
+                        # in which case we should shift it like its base
+                        # line got shifted.
+                        for j in range(i - 1, -1, -1):
+                            jline, jlevel = stats[j]
+                            if jlevel >= 0:
+                                want = have + _getlspace(after[jline - 1]) - \
+                                       _getlspace(lines[jline])
+                                break
+                    if want < 0:
+                        # Still no luck -- leave it alone.
+                        want = have
+                else:
+                    want = 0
+            assert want >= 0
+            have2want[have] = want
+            diff = want - have
+            if diff == 0 or have == 0:
+                after.extend(lines[thisstmt:nextstmt])
+            else:
+                for line in lines[thisstmt:nextstmt]:
+                    if diff > 0:
+                        if line == "\n":
+                            after.append(line)
+                        else:
+                            after.append(" " * diff + line)
+                    else:
+                        remove = min(_getlspace(line), -diff)
+                        after.append(line[remove:])
+        return self.raw != self.after
+
+    def fixed_lines(self):
+        return self.after
+
+    def getline(self):
+        """Line-getter for tokenize."""
+        if self.index >= len(self.lines):
+            line = ""
+        else:
+            line = self.lines[self.index]
+            self.index += 1
+        return line
+
+    def tokeneater(self, token_type, _, start, __, line,
+                   INDENT=tokenize.INDENT,
+                   DEDENT=tokenize.DEDENT,
+                   NEWLINE=tokenize.NEWLINE,
+                   COMMENT=tokenize.COMMENT,
+                   NL=tokenize.NL):
+        """Line-eater for tokenize."""
+        sline = start[0]
+        if token_type == NEWLINE:
+            # A program statement, or ENDMARKER, will eventually follow,
+            # after some (possibly empty) run of tokens of the form
+            #     (NL | COMMENT)* (INDENT | DEDENT+)?
+            self.find_stmt = 1
+
+        elif token_type == INDENT:
+            self.find_stmt = 1
+            self.level += 1
+
+        elif token_type == DEDENT:
+            self.find_stmt = 1
+            self.level -= 1
+
+        elif token_type == COMMENT:
+            if self.find_stmt:
+                self.stats.append((sline, -1))
+                # but we're still looking for a new stmt, so leave
+                # find_stmt alone
+
+        elif token_type == NL:
+            pass
+
+        elif self.find_stmt:
+            # This is the first "real token" following a NEWLINE, so it
+            # must be the first token of the next program statement, or an
+            # ENDMARKER.
+            self.find_stmt = 0
+            if line:   # not endmarker
+                self.stats.append((sline, self.level))
+
+
+def _getlspace(line):
+    """Count number of leading blanks."""
+    i = 0
+    while i < len(line) and line[i] == " ":
+        i += 1
+    return i
+
+
 def fix_file(filename, opts):
     tmp_source = read_from_filename(filename)
     fix = FixPEP8(filename, opts, contents=tmp_source)
@@ -670,8 +846,10 @@ def main():
                       help='diff print of fixed source.')
     parser.add_option('-i', '--in-place', action='store_true',
                       help='make changes to files in place')
-    parser.add_option('-p', '--pep8-passes', default=20, type='int',
-                      help='maximum number of additional pep8 passes')
+    parser.add_option('-p', '--pep8-passes',
+                      default=PEP8_PASSES_MAX, type='int',
+                      help='maximum number of additional pep8 passes'
+                           ' (default:%d)' % PEP8_PASSES_MAX)
     parser.add_option('--ignore', default='',
                       help='do not fix these errors/warnings (e.g. E4,W)')
     opts, args = parser.parse_args()
