@@ -4,7 +4,6 @@ import unittest
 from subprocess import Popen, PIPE
 from tempfile import mkstemp
 
-import sys
 sys.path.insert(0,
                 os.path.split(os.path.abspath(os.path.dirname(__file__)))[0])
 import autopep8
@@ -64,6 +63,44 @@ class TestFixPEP8Error(unittest.TestCase):
         p = Popen(cmd, stdout=PIPE)
         self.result = p.communicate()[0].decode('utf8')
 
+    def test_e101(self):
+        line = """
+while True:
+    if True:
+    \t1
+""".lstrip()
+        fixed = """
+while True:
+    if True:
+        1
+""".lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e101_with_comments(self):
+        line = """
+while True:  # My inline comment
+             # with a hanging
+             # comment.
+    # Hello
+    if True:
+    \t# My comment
+    \t1
+    \t# My other comment
+""".lstrip()
+        fixed = """
+while True:  # My inline comment
+             # with a hanging
+             # comment.
+    # Hello
+    if True:
+        # My comment
+        1
+        # My other comment
+""".lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e111_short(self):
         line = "class Dummy:\n  def __init__(self):\n    pass\n"
         fixed = "class Dummy:\n    def __init__(self):\n        pass\n"
@@ -76,17 +113,94 @@ class TestFixPEP8Error(unittest.TestCase):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e111_longer(self):
+        line = """
+while True:
+      if True:
+            1
+      elif True:
+            2
+""".lstrip()
+        fixed = """
+while True:
+    if True:
+        1
+    elif True:
+        2
+""".lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e111_multiple_levels(self):
         line = """
 while True:
     if True:
        1
-"""
+
+# My comment
+print('abc')
+
+""".lstrip()
         fixed = """
 while True:
     if True:
         1
-"""
+
+# My comment
+print('abc')
+""".lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e111_with_dedent(self):
+        line = """
+def foo():
+    if True:
+         2
+    1
+""".lstrip()
+        fixed = """
+def foo():
+    if True:
+        2
+    1
+""".lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e111_with_other_errors(self):
+        line = """
+def foo():
+    if True:
+         (2 , 1)
+    1
+    if True:
+           print('hello')  
+    2
+""".lstrip()
+        fixed = """
+def foo():
+    if True:
+        (2, 1)
+    1
+    if True:
+        print('hello')
+    2
+""".lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e191(self):
+        line = """
+while True:
+\tif True:
+\t\t1
+""".lstrip()
+        fixed = """
+while True:
+    if True:
+        1
+""".lstrip()
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
@@ -144,6 +258,20 @@ while True:
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e223_with_tab_indentation(self):
+        line = """
+class Foo():
+\tdef __init__(self):
+\t\tx= 1\t+ 3
+""".lstrip()
+        fixed = """
+class Foo():
+\tdef __init__(self):
+\t\tx = 1 + 3
+""".lstrip()
+        self._inner_setup(line, options='--ignore=W191')
+        self.assertEqual(self.result, fixed)
+
     def test_e224(self):
         line = "a = 11 +	1\n"    # include TAB
         fixed = "a = 11 + 1\n"
@@ -156,12 +284,32 @@ while True:
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e224_with_tab_indentation(self):
+        line = """
+class Foo():
+\tdef __init__(self):
+\t\tx= \t3
+""".lstrip()
+        fixed = """
+class Foo():
+\tdef __init__(self):
+\t\tx = 3
+""".lstrip()
+        self._inner_setup(line, options='--ignore=W191')
+        self.assertEqual(self.result, fixed)
+
     def test_e225(self):
         line = "1+1\n2 +2\n3+ 3\n"
         fixed = "1 + 1\n2 + 2\n3 + 3\n"
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e231(self):
+        line = "[1,2,3]\n"
+        fixed = "[1, 2, 3]\n"
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e241(self):
         line = "l = (1,  2)\n"
         fixed = "l = (1, 2)\n"
@@ -282,6 +430,14 @@ while True:
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e702_skip_with_triple_quote(self):
+        # We do not support this yet.
+        # We would expect '"""\n      hello\n   """\n1\n'.
+        line = '"""\n      hello\n   """;1\n'
+        fixed = line
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
 
 class TestFixPEP8Warning(unittest.TestCase):
 
@@ -515,6 +671,14 @@ class TestFixPEP8Warning(unittest.TestCase):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    @only_py2
+    def test_w602_raise_argument_triple_skip_with_indentation(self):
+        # We do not handle this properly yet.
+        line = 'if True:\n    raise TypeError, "error", tb\n'
+        fixed = line
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     @only_py2
     def test_w602_raise_argument_triple_with_comment(self):
         line = 'raise ValueError, "info", traceback  # comment\n'
@@ -544,8 +708,16 @@ class TestFixPEP8Warning(unittest.TestCase):
     def test_w604_skip_multiple_instances(self):
         # We do not support this yet
         line = "``1`` + ``b``\n"
+        fixed = line
         self._inner_setup(line)
-        self.assertEqual(self.result, line)
+        self.assertEqual(self.result, fixed)
+
+    def test_w604_skip_multiple_lines(self):
+        # We do not support this yet
+        line = "`(1\n)`\n"
+        fixed = line
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
 
 
 class TestOptions(unittest.TestCase):
